﻿//+--------------------------------------------------------------------------+
//|                                                                          |
//|                               Carousel                                   |
//|                         GDImage 3D animation                             |
//|                                                                          |
//|          The GDImage.dll trial version has no time limitation,           |
//|            and all features of the API are fully functional.             |
//|                                                                          |
//|      Buying a commercial license removes the copyright nag screen.       |
//|                                                                          |
//| L'utilisation à titre privé de la version d'évaluation de GDImage n'est  |
//|   pas limitée dans le temps et toutes les fonctions sont disponibles.    |
//|                                                                          |
//|              L'acquisition d'une licence permet de supprimer             |
//|                       l'affichage du copyright.                          |
//|                                                                          |
//|                             Version 1.00                                 |
//+--------------------------------------------------------------------------+
//|                                                                          |
//|                         Author Patrice TERRIER                           |
//|                           copyright (c) 2006                             |
//|                                                                          |
//|                        pterrier@zapsolution.com                          |
//|                                                                          |
//|                          www.zapsolution.com                             |
//|                                                                          |
//+--------------------------------------------------------------------------+
//|                  Project started on : 01-15-2007 (MM-DD-YYYY)            |
//|                        Last revised : 01-25-2007 (MM-DD-YYYY)            |
//+--------------------------------------------------------------------------+

using System;
using System.Text;
using System.Windows.Forms;
using System.Drawing;
using System.Runtime.InteropServices;
using System.Threading;

using Win32;
using SkinEngine;
using GDImage;

namespace Carousel
{
    [UnmanagedFunctionPointer(CallingConvention.StdCall)]
    public delegate bool CallBack(IntPtr hWnd, int wMsg, int wParam, int lParam);

    public partial class MAIN_Form : Form
    {
        const int TIMER_MAX = 2;                         // To perform smooth animation
        const int ID_CTRL = 100;                         // Main GDImage control

        const int ID_OBJECT_TEXT = 9;                    // Each of the sprite must use a unique ID
	    const int ID_OBJECT_SPRITE	= 10;
	
	    const double PiDiv180 = 1.74532925199433E-2;

        static public IntPtr hFORM_Main = IntPtr.Zero;   // Default to Zero
        static public IntPtr hGDImageCtrl = IntPtr.Zero; // Default to Zero
        static public IntPtr hBackGround = IntPtr.Zero;  // Default to Zero

        public struct RECT { public int left, top, right, bottom;}

        public struct SpriteDataStruct
        {
            public string ImageName;
            public IntPtr hBitmap;
	        public int ID;
	        public int xPos;
	        public int yPos;
	        public int nWidth;
	        public int nHeight;
        }

        static SpriteDataStruct[] SpriteData = new SpriteDataStruct[10];
        static Single[] SpriteAngle = new Single[10];
        static int[] Zorder = new int[10];
        static int[] FaceOrder = new int[10];
        static int gnBoundWidth, gnBoundHeight;
        static bool gbMoveToLeft = false;
        static bool gbAnimate = false;

        static double Focal_InitialValue;
        static double Speed_InitialValue;
        static double Y_InitialValue;

        // Very important the call back must be static
        // to prevent from access violation during garbage collection.
        // Search for <<Callback Functions>>
        static CallBack UseCallBack = null;

        public MAIN_Form()
        {

            // We use a callback to handle the GDImage events         
            // Very important the call back must be static
            // to prevent from access violation during garbage collection.
            // Search for <<Callback Functions>>
            UseCallBack = new CallBack(GDImageCallBack);
           
            hFORM_Main = this.Handle;

            this.SetStyle(ControlStyles.DoubleBuffer, true);
            this.SetStyle(ControlStyles.AllPaintingInWmPaint, true);
            this.SetStyle(ControlStyles.UserPaint, true);
            this.SetStyle(ControlStyles.ResizeRedraw, false);
            //this.SetStyle(ControlStyles.SupportsTransparentBackColor, true);
            ////this.SetStyle(ControlStyles.StandardDoubleClick, false);
            ////this.SetStyle(ControlStyles.Selectable, true);
            //this.ResizeRedraw = true;

            // Setup Skin parameters
            SetSkinParameters();

            InitializeComponent();

            GDImage_Template.Visible = false;
            GDImage_Template.Enabled = false;
            SaveInitialValue();

            // Create the GDImage control using SDK style
            uint Style = Api.WS_CHILD | Api.WS_VISIBLE;
            uint StyleEx = 0;
            hGDImageCtrl = Api.CreateWindowEx(StyleEx,
                           GI.GDImageClassName,             // window class name
                           "",                              // window title
                           Style,                           // window style
                           GDImage_Template.Location.X,     // x location
                           GDImage_Template.Location.Y,     // y location
                           GDImage_Template.Size.Width,     // Width
                           GDImage_Template.Size.Height,    // Height
                           GDImage_Template.Parent.Handle,  // parent window handle
                           ID_CTRL,                         // ControlID
                           Api.GetModuleHandle(""),         // program instance handle
                           0);                              // creation parameters

            if (hGDImageCtrl != IntPtr.Zero)
            {
                GI.SetProperty(hGDImageCtrl, GI.ZI_GradientTop, Api.RGB(27, 29, 41));
                GI.SetProperty(hGDImageCtrl, GI.ZI_GradientBottom, Api.RGB(27, 29, 41));
            }

        }

        private void SetSkinParameters()
        {
            // REM this out to disable the drop shadow effect.
            //Api.UseDropShadow(hFORM_Main);

            // Setup default Skin colors
            SK.ColorLayerBackground = Color.FromArgb(192, 192, 202);
            SK.ColorCaptionEnabled = Color.FromArgb(134, 153, 190);
            SK.ColorCaptionDisabled = Color.FromArgb(73, 76, 99);
            SK.TooltipBackColor = Color.FromArgb(73, 76, 99);
            SK.TooltipForeColor = Color.FromArgb(134, 153, 190);

            // Skin opacity in the range 0-255
            SK.Alpha = 255;//204; // 80 % default transparency level

            // Skin opacity of the hidden background in the range 0-254
            //SK.OpacityLayerBackground = 248;

            //SK.IsResizable = false;
        }

        private void MonitorGDImageEvents()
        {
            UseCallBack = new CallBack(GDImageCallBack);

            // We use a callback to monitor the GDImage control messages
            // Create a %WM_LBUTTONDOWN event
            GI.ZI_EventMessage(GDImageCallBack, Api.WM_LBUTTONDOWN, true);
            // Create a WM_RBUTTONDOWN event
            GI.ZI_EventMessage(GDImageCallBack, Api.WM_RBUTTONDOWN, true);
            // Create a WM_KEYDOWN event
            GI.ZI_EventMessage(GDImageCallBack, Api.WM_KEYDOWN, true);
            // Create a WM_MOUSEMOVE event
            GI.ZI_EventMessage(GDImageCallBack, Api.WM_MOUSEMOVE, true);

            GC.KeepAlive(UseCallBack);
        }

        private void LoadSprites()
        {
            string ExePath = Api.ExePath();
            SpriteData[0].ImageName = ExePath + "guide.png";
            SpriteData[1].ImageName = ExePath + "photoshop.png";
            SpriteData[2].ImageName = ExePath + "mac.png";
            SpriteData[3].ImageName = ExePath + "msn.png";
            SpriteData[4].ImageName = ExePath + "quicktime.png";
            SpriteData[5].ImageName = ExePath + "fileprint.png";
            SpriteData[6].ImageName = ExePath + "mail.png";
            SpriteData[7].ImageName = ExePath + "ecran.png";
            SpriteData[8].ImageName = ExePath + "babelfish.png";
            SpriteData[9].ImageName = ExePath + "burner.png";

            int bmW, bmH;
            int AngleStep = 0;
            int UboundSpriteData = SpriteData.Length;
            int StepAngle = (int)(360 / UboundSpriteData);
            int Item;

            for (int K = 0; K < UboundSpriteData; K++)
            {
                Item = ID_OBJECT_SPRITE + K + 1;

                //SpriteData[K].hBitmap = GI.ZI_CreateMirrorBitmapFromFile(SpriteData[K].ImageName, out bmW, out bmH);
                if (SWITCH_SelfRotation.SwitchStatus)
                {
                    SpriteData[K].hBitmap = GI.ZI_CreateBitmapFromFile(SpriteData[K].ImageName, out bmW, out bmH); }
                else {
                    SpriteData[K].hBitmap = GI.ZI_CreateMirrorBitmapFromFile(SpriteData[K].ImageName, out bmW, out bmH);
                }

                SpriteData[K].ID = Item;
                SpriteData[K].nWidth = bmW;
                SpriteData[K].nHeight = bmH;
                SpriteAngle[K] = AngleStep;

                GI.ZD_DrawBitmapToCtrl(hGDImageCtrl,
                                       SpriteData[K].xPos, SpriteData[K].yPos,
                                       SpriteData[K].hBitmap,
                                       GI.ZD_ColorARGB(255, Api.RGB(255, 255, 255)),
                                       SpriteData[K].ID,
                                       GI.ZS_VISIBLE);

                // Throw it out of the control view port
                GI.ZD_SetObjectXY(Item, -4000, -4000, GI.ZD_DRAW_DEFERRED);

                GI.ZD_UsePngOpacity(Item, true);

                AngleStep += StepAngle;

            }
            uint ARGB = GI.ZD_ColorARGB(128, Api.RGB(255, 255, 255));
            string MarqueeMsg = "No DirectX, no OpenGL, no WPF, just plain lean GDImage ...";
            string FontToUse = "Times New Roman";
            GI.ZD_DrawTextToCtrl(hGDImageCtrl,
            MarqueeMsg, 					// The text to be displayed
            GDImage_Template.Size.Width, 	// X coordinate
            10, 							// Y coordinate
            ARGB, 			    			// ARGB color to use
            FontToUse, 			    		// The True Type Font to use (must be a valid one)
            40, 							// The font size in pixel
            ID_OBJECT_TEXT, 				// The unique object ID
            GI.ZS_VISIBLE, 					// Overlay visible at startup
            0, 						    	// Optional shadow effect (offset in pixel)
            0);                             // Optional string format

            GI.ZD_GetObjectBound(ID_OBJECT_TEXT, out gnBoundWidth, out gnBoundHeight);

        }

        private void FORM_Load(object sender, EventArgs e)
        {
            // We need this to retrieve bitmap from resource
            SK.UseNameSpace = this.GetType().Namespace;

            // Use bitmap resources to setup public value for:
            // FixCaptionLeft, FixCaptionHeight, FixCaptionRight, FixBottomHeight.
            Bitmap bmp;
            bmp = Properties.Resources.TopLeft;
            SK.FixCaptionLeft = bmp.Width; SK.FixCaptionHeight = bmp.Height;
            bmp = Properties.Resources.BottomRight;
            SK.FixCaptionRight = bmp.Width; SK.FixBottomHeight = bmp.Height;
            SK.DockMin = SK.FixCaptionHeight + SK.FixBottomHeight; // Docking

            // Setup the minimum size of the form
            int UseMinWidth = Math.Max(this.MinimumSize.Width, SK.FixCaptionHeight + SK.FixBottomHeight);
            int UseMinHeight = Math.Max(this.MinimumSize.Height, SK.FixCaptionHeight + SK.FixBottomHeight);
            SK.UnDockingHeight = UseMinHeight; // Save the UnDockingHeight
            this.MinimumSize = new System.Drawing.Size(UseMinWidth, UseMinHeight);

            this.BackgroundImage = (Bitmap)SK.DrawBackGround(this.Width, this.Height);

            Bitmap bmpFrmBack = (Bitmap)this.BackgroundImage;
            SK.CreateFormRegion(this, bmpFrmBack);
            //RECT rw = new RECT();
            Api.RECT rw = new Api.RECT();
            Api.GetWindowRect(hFORM_Main, ref rw);
            SK.LastRgnWidth = rw.right - rw.left; SK.LastRgnHeight = rw.bottom - rw.top;

            // Status text color
            //STATUS_Label.ForeColor = SK.ColorCaptionEnabled;

            // Caption text + color
            if (this.Enabled)
            { CAPTION_Label.ForeColor = SK.ColorCaptionEnabled; }
            else
            { CAPTION_Label.ForeColor = SK.ColorCaptionDisabled; }
            CAPTION_Label.TextAlign = ContentAlignment.MiddleLeft;
            CAPTION_Label.Text = this.Text;

            // FORM_Tooltip colors
            FORM_Tooltip.BackColor = SK.TooltipBackColor;
            FORM_Tooltip.ForeColor = SK.TooltipForeColor;

            // Check first if we can use layered window.
            byte Alpha = 0;
            if ((Api.GetOsVersion() > 499) && (SK.Alpha < 255))
            {
                Alpha = 254; this.Opacity = (double)Alpha / 255;
            }
            else
            {
                Alpha = 255; this.Opacity = (double)Alpha / 255;
            }
            // Support for layered window is a prerequisite to create aero glass effect
            if (Alpha < 255)
            {
                Api.SetLayeredWindowAttributes(hFORM_Main, 0, SK.Alpha, 0x00000002);
            //    //Create the skin background
            //    Form BackGround = new Form();
            //    BackGround.ShowInTaskbar = false; // WS_EX_TOOLWINDOW
            //    BackGround.Visible = true;
            //    BackGround.FormBorderStyle = 0;
            //    BackGround.BackColor = SK.ColorLayerBackground;
            //    BackGround.Opacity = 0.999; // Don't change this !
            //    hBackGround = BackGround.Handle;
            //    Api.SetLayeredWindowAttributes(hBackGround, 0, SK.OpacityLayerBackground, 0x00000002);
            //    ResizeMoveHiddenWindows();
            }

        }

        // ****************************************
        // Callback that handles the GDImage events
        // Search for <<Callback Functions>>
        public unsafe static bool GDImageCallBack(IntPtr hWnd, int wMsg, int wParam, int lParam)
        {
            Boolean bRet = false; // Do not stop the event processing in GDImage
            string LabelIs;
            //string sMessage;
            int ObjectID = 0;
            int x, y, x1, y1, useWidth, useHeight, BoundWidth, BoundHeight,
                x2Div2, y2Div2, UseStep;
            Api.RECT rc = new Api.RECT();

            if (hWnd == hGDImageCtrl)
            {
                switch (wMsg)
                {
                    case Api.WM_LBUTTONDOWN:
                        ObjectID = GI.ZI_MouseOverObjectID();
                        if (ObjectID == Api.GetDlgCtrlID(hWnd))
                        {
                            LabelIs = "Background";
                        }
                        else
                        {
                            LabelIs = GI.ZD_GetObjectImageLabel(ObjectID);
                        }
                        //sMessage = @"WM_LBUTTONDOWN on object" + ObjectID.ToString() +
                        //           " >" + LabelIs + "<";
                        //Api.SetWindowText(Api.GetDlgItem(hFORM_Main, ID_STATUS_Main), sMessage);
                        break;

                    case Api.WM_RBUTTONDOWN:
                        ObjectID = GI.ZI_MouseOverObjectID();
                        if (ObjectID == Api.GetDlgCtrlID(hWnd))
                        {
                            LabelIs = "Background";
                        }
                        else
                        {
                            LabelIs = GI.ZD_GetObjectImageLabel(ObjectID);
                        }
                        //sMessage = @"WM_RBUTTONDOWN on object" + ObjectID.ToString() +
                        //           " >" + LabelIs + "< at location " +
                        //           Api.LoWrd(lParam).ToString() + "," +
                        //           Api.HiWrd(lParam).ToString();
                        //Api.SetWindowText(Api.GetDlgItem(hFORM_Main, ID_STATUS_Main), sMessage);
                        break;

                    case Api.WM_MOUSEMOVE:
                        ObjectID = GI.ZI_GetMovingSpriteID();
                        if (ObjectID != 0)
                        {
                        }
                        break;

                    case Api.WM_KEYDOWN:
                        ObjectID = GI.ZI_GetObjectFocusID();
                        if (ObjectID != 0)
                        {
                            GI.ZD_GetObjectXY(ObjectID, out x, out y);
                            x1 = x; // Make a copy to keep the orignal x location unchanged
                            y1 = y; // Make a copy to keep the orignal y location unchanged 

                            // Check accelerator keys to compute the step range 
                            if (GI.ZI_IsCtrlKeyPressed())
                            {
                                UseStep = 4;
                                if (GI.ZI_IsShiftKeyPressed()) UseStep = 16;
                            }
                            else
                            {
                                UseStep = 1;
                                if (GI.ZI_IsShiftKeyPressed()) UseStep = 2;
                            }

                            // If object scroll with the bitmap background
                            if (GI.ZD_GetObjectScroll(ObjectID))
                            {
                                // Get the size of the bitmap background
                                GI.ZI_GetBitmapSize(GI.ZI_GetBMP(Api.GetDlgItem(hFORM_Main, ID_CTRL)), out useWidth, out useHeight);
                            }
                            else
                            {
                                // Get the control client size  
                                Api.GetClientRect(Api.GetDlgItem(hFORM_Main, ID_CTRL), ref rc);
                                useWidth = rc.right; useHeight = rc.bottom;
                            }
                            // Get the sprite object size
                            GI.ZD_GetObjectBound(ObjectID, out BoundWidth, out BoundHeight);

                            x2Div2 = (int)(BoundWidth / 2); y2Div2 = (int)BoundHeight / 2;

                            switch (wParam)
                            {
                                case Api.VK_HOME:
                                    x1 = 0;
                                    break;
                                case Api.VK_END:
                                    x1 = Math.Max(useWidth - BoundWidth, 0);
                                    break;
                                case Api.VK_PRIOR:
                                    y1 = 0;
                                    break;
                                case Api.VK_NEXT:
                                    y1 = Math.Max(useHeight - BoundHeight, 0);
                                    break;
                                case Api.VK_NUMPAD4:
                                case Api.VK_LEFT:
                                    if (x1 > -x2Div2) x1 = Math.Max(x1 - UseStep, -x2Div2);
                                    break;
                                case Api.VK_NUMPAD8:
                                case Api.VK_UP:
                                    if (y1 > -y2Div2) y1 = Math.Max(y1 - UseStep, -y2Div2);
                                    break;
                                case Api.VK_NUMPAD6:
                                case Api.VK_RIGHT:
                                    if (x1 < useWidth - x2Div2)
                                    {
                                        x1 = Math.Min(x1 + UseStep, useWidth - x2Div2);
                                    }
                                    break;
                                case Api.VK_NUMPAD2:
                                case Api.VK_DOWN:
                                    if (y1 < useHeight - y2Div2)
                                    {
                                        y1 = Math.Min(y1 + UseStep, useHeight - y2Div2);
                                    }
                                    break;
                                default:
                                    break;
                            }

                            if ((x != x1) || (y != y1))
                            {
                                x = x1; y = y1;
                                GI.ZD_SetObjectXY(ObjectID, x1, y1, GI.ZD_DRAW_REDRAW);
                            }
                            //sMessage = "Object " + ObjectID.ToString() + " coordinates " + x.ToString() + "," + y.ToString();
                            //Api.SetWindowText(Api.GetDlgItem(hFORM_Main, ID_STATUS_Main), sMessage);
                            bRet = true;
                        }
                        break;
                }
            }
            return bRet;
        }

        private void FORM_Close(object sender, EventArgs e)
        {
            this.Dispose();
        }

        //***********************************************************
        // We use the .NET template to resize the GDImage control
        private void FORM_Main_Resize(object sender, EventArgs e)
        {
            Api.MoveWindow(hGDImageCtrl, GDImage_Template.Location.X, GDImage_Template.Location.Y, GDImage_Template.Size.Width, GDImage_Template.Size.Height, true);
        }

        private void FOCUS_Activated(object sender, EventArgs e)
        {
            if (Api.IsWindow(hBackGround))
            {
                uint swpStyle = Api.SWP_NOSIZE | Api.SWP_NOMOVE | Api.SWP_NOSENDCHANGING | Api.SWP_NOOWNERZORDER | Api.SWP_NOACTIVATE | Api.SWP_NOREDRAW;
                Api.SetWindowPos(hBackGround, IntPtr.Zero, 0, 0, 0, 0, swpStyle);
                swpStyle = Api.SWP_NOSIZE | Api.SWP_NOMOVE | Api.SWP_NOSENDCHANGING;
                Api.SetWindowPos(hFORM_Main, IntPtr.Zero, 0, 0, 0, 0, swpStyle);
            }
            if (LED_Off.Visible)
            {
                CAPTION_Label.ForeColor = SK.ColorCaptionEnabled;
                LED_On.Visible = true;
                LED_Off.Visible = false;
            }
        }

        private void SwitchOff()
        {
            CAPTION_Label.Select();
            CAPTION_Label.ForeColor = SK.ColorCaptionDisabled;
            LED_On.Visible = false;
            LED_Off.Visible = true;
        }

        private void FOCUS_Deactivate(object sender, EventArgs e)
        {
            SwitchOff();
        }

        protected override void WndProc(ref Message m)
        {

            // We monitor the GDImage events from there
            // And we are doing it from there to protect us from the infamous
            // CallbackOnCollectedDelegate
            // (rappel effectué sur un délégué par le garbage collector)
            MonitorGDImageEvents();

            switch (m.Msg)
            {
                case Api.WM_SIZE:

                    //Start flicker free section
                    StopAnimation();
//                  Api.ShowWindow(Skin_STATUS_Dark.Handle, Api.SW_HIDE);

                    SK.BuildRegion(m.HWnd);

                    //End flicker free section
//                  Api.ShowWindow(Skin_STATUS_Dark.Handle, Api.SW_SHOW);
                    StartAnimation();

                    m.Result = (IntPtr)0;
                    return;

                case Api.WM_NCLBUTTONDBLCLK:
                    // We handle this ourself to avoid havoc when using restore
                    // when using a form without caption.
                    if (SK.IsMaximized)
                    {
                        //Api.ButtonClick(BTN_Restore.Handle);
                        FORM_Restore();
                    }
                    else
                    {
                        //Api.ButtonClick(BTN_Maximize.Handle);
                        FORM_Maximize();
                    }
                    m.Result = (IntPtr)0;
                    return;

                case Api.WM_LBUTTONDBLCLK:
                    if (SK.IsMaximized)
                    { // We simulate double click on caption to restore the window size
                        if (Api.HiWrd((uint)m.LParam) < SK.FixCaptionHeight) Api.ButtonClick(BTN_Restore.Handle);
                    }
                    break;

                case Api.WM_ACTIVATEAPP: // Keep hidden background behind the form
                    if (Api.IsWindow(hBackGround))
                    {
                        if ((uint)m.WParam == 0)
                        {
                            uint swpStyle = Api.SWP_NOSIZE | Api.SWP_NOMOVE | Api.SWP_NOSENDCHANGING | Api.SWP_NOOWNERZORDER | Api.SWP_NOACTIVATE | Api.SWP_NOREDRAW;
                            Api.SetWindowPos(hBackGround, Api.GetWindow(hFORM_Main, 2), 0, 0, 0, 0, swpStyle);
                        }
                    }
                    break;

                case Api.WM_COMMAND:
                    // I am using IF there 
                    // because C# doesn't handle IntPtr in a SWITCH select case
                    if (m.LParam == SWITCH_Wallpaper.Handle)
                    { Wallpaper_Click(); }
                    else if (m.LParam == SWITCH_Opacity.Handle)
                    { }
                    else if (m.LParam == SWITCH_SelfRotation.Handle)
                    {
                        int bmW, bmH;
                        int UboundSpriteData = SpriteData.Length;
                        bool UseRotation = SWITCH_SelfRotation.SwitchStatus;
                        for (int K = 0; K < UboundSpriteData; K++)
                        {
                            if (UseRotation) {
                                SpriteData[K].hBitmap = GI.ZI_CreateBitmapFromFile(SpriteData[K].ImageName, out bmW, out bmH); }
                            else {
                                SpriteData[K].hBitmap = GI.ZI_CreateMirrorBitmapFromFile(SpriteData[K].ImageName, out bmW, out bmH);
                            }
                            GI.ZD_DrawBitmapToCtrl(hGDImageCtrl,
                                                   SpriteData[K].xPos, SpriteData[K].yPos,
                                                   SpriteData[K].hBitmap,
                                                   GI.ZD_ColorARGB(255, 0),
                                                   SpriteData[K].ID,
                                                   GI.ZS_VISIBLE);

                        }
                    }
                    else if (m.LParam == SWITCH_Marquee.Handle)
                    {
                        if (SWITCH_Marquee.SwitchStatus)
                        {
                            GI.ShowObject(ID_OBJECT_TEXT, GI.ZD_SHOW);
                        }
                        else
                        {
                            GI.ShowObject(ID_OBJECT_TEXT, GI.ZD_HIDE);
                        }
                    }
                    break;

                case Api.WM_TIMER:
                    int wParam = (int)m.WParam;
                    if (wParam == 1)
                    {
                        DrawCarousel();
                    }
                    m.Result = (IntPtr)0;
                    return;

                case Api.WM_DESTROY:
                    StopAnimation();
                    break;
            }

            base.WndProc(ref m);

            // This is the place where the skin NC area is handled.
            if (m.Msg == Api.WM_NCHITTEST)
            {
                if (SK.IsMaximized) return;

                if (m.Result == (IntPtr)Api.HTCLIENT)
                {
                    int HITTEST = Api.HTCAPTION;
                    if (SK.IsResizable) // Check if the form is resizable
                    {
                        Point p = this.PointToClient(new Point(Api.LoWrd((uint)m.LParam), Api.HiWrd((uint)m.LParam)));

                        int xF = this.Width, yF = this.Height;
                        int xSide = 0;
                        int Border = System.Windows.Forms.SystemInformation.Border3DSize.Width;
                        if ((p.X >= xF - SK.FixBottomHeight) && ((p.Y >= yF - SK.FixBottomHeight)))
                        {
                            HITTEST = Api.HTBOTTOMRIGHT;
                        }
                        else
                        {
                            // Left side
                            if (p.X <= 8)
                            {
                                if (p.X <= Border) HITTEST = Api.HTLEFT;
                                xSide = 1;
                            }
                            // Right side
                            if (p.X >= xF - 8)
                            {
                                if (p.X >= xF - Border) HITTEST = Api.HTRIGHT;
                                xSide = 2;
                            }
                            // Top side
                            if (p.Y <= Border)
                            {
                                HITTEST = Api.HTTOP;
                                if (xSide == 1)
                                {
                                    HITTEST = Api.HTTOPLEFT;
                                }
                                else if (xSide == 2)
                                {
                                    HITTEST = Api.HTTOPRIGHT;
                                }
                            }
                            // Bottom side
                            if (p.Y >= yF - Border)
                            {
                                if (xSide == 1)
                                {
                                    HITTEST = Api.HTBOTTOMLEFT;
                                }
                                else
                                {
                                    HITTEST = Api.HTBOTTOM;
                                }
                            }
                        }
                    }

                    m.Result = (IntPtr)HITTEST;
                }
            }
        }

        private void BTN_Close_Click(object sender, EventArgs e)
        {
            Close();
        }

        private void BTN_Iconize_Click(object sender, EventArgs e)
        {
            this.WindowState = FormWindowState.Minimized;
        }

        private int WasX, WasY, WasWidth, WasHeight;
        private void FORM_Restore()
        {
            //CLIENT_Panel.Visible = false;

            BTN_Maximize.Visible = true;
            BTN_Restore.Visible = false;

            bool RedrawMode = true;
            if (SK.Alpha < 255)
            {
                Api.SetLayeredWindowAttributes(hFORM_Main, 0, SK.Alpha, 0x00000002);
                RedrawMode = false;
                if (Api.IsWindow(hBackGround)) Api.MoveWindow(hBackGround, WasX, WasY, WasWidth, WasHeight, false);
            }
            Api.MoveWindow(hFORM_Main, WasX, WasY, WasWidth, WasHeight, RedrawMode);

            SK.IsMaximized = false;
        }
        private void BTN_Restore_Click(object sender, EventArgs e)
        {
            FORM_Restore();
        }

        private void FORM_Maximize()
        {
            WasX = this.Location.X;
            WasY = this.Location.Y;
            WasWidth = this.Width;
            WasHeight = this.Height;

            SK.IsMaximized = true;

            Api.ControlDrawingDisable(this.Handle);

            bool RedrawMode = true;
            if (SK.Alpha < 255)
            {
                RedrawMode = false;
                // Turn it to opaque mode
                Api.SetLayeredWindowAttributes(hFORM_Main, 0, 255, 0x00000002);
                if (Api.IsWindow(hBackGround)) Api.MoveWindow(hBackGround, -9990, -9990, WasWidth, WasHeight, false);
            }

            Api.MoveWindow(hFORM_Main, 0, 0, SystemInformation.WorkingArea.Width, SystemInformation.WorkingArea.Height, RedrawMode);

            BTN_Restore.Visible = true;
            BTN_Maximize.Visible = false;

            Api.ControlDrawingEnable(this.Handle);
        }
        private void BTN_Maximize_Click(object sender, EventArgs e)
        {
            FORM_Maximize();
        }

        private void StartAnimation()
        {
            // Use API WM_TIMER to avoid aggressive garbage collection 
            // to occur before Main ends.
            Api.SetTimer(hFORM_Main, 1, 0, null);
        }

        private void StopAnimation()
        {
            Api.KillTimer(hFORM_Main, 1);
        }
        
        private void DrawCarousel()
        {
            // Based on 
            if (gbAnimate)
            {
                int Y        = (int)POT_Y.Value;              // Default = 50
                float speed  = (float)POT_Speed.Value / 4;    // Default = 4 
                float fl     = (float)POT_Focal.Value / 100;  // Default = 15000
                             if (fl < 0.02f) fl = 0.001f;
                float Radius = 160.0f;
                int Xcenter  = (GDImage_Template.Size.Width - 128) / 2;
                int Ycenter  = (GDImage_Template.Size.Height - 128) / 2;
                int Zcenter  = Ycenter / 2;
        
                int UboundSpriteData = SpriteData.Length;
                int K;
        
                for (K = UboundSpriteData - 1; K > -1; K--)
                {
                    float angle = SpriteAngle[K];
                    float anglePi = (float)(angle * PiDiv180);
                    float z = (float)(Math.Sin(anglePi) * Radius + Zcenter);
                    float scale = fl / (fl + z);
                    if (fl == 0.001f) { 
                        SpriteData[K].xPos = Xcenter;
                        SpriteData[K].yPos = Ycenter + Y; }
                    else  {
                        int X = (int)(Math.Cos(anglePi) * Radius);
                        SpriteData[K].xPos = (int)(X * scale + Xcenter);
                        SpriteData[K].yPos = (int)(Y * scale + Ycenter - Y);
                    }

                    Zorder[K] = (int)(angle - 90); if (Zorder[K] > 180) Zorder[K] = 360 - Zorder[K];
                    FaceOrder[K] = K;
                
                    if (gbMoveToLeft)
                    {
                        angle -= speed; if (angle < 1) angle = 360;
                    }
                    else
                    {
                        angle += speed; if (angle > 359) angle = angle - 360;
                    }

                    SpriteAngle[K] = angle;

                    //  Compute the scale factor
                    if (SWITCH_Use3D.SwitchStatus)
                    {
                        GI.ZD_SetObjectScale(SpriteData[K].ID, (Math.Abs(Zorder[K]) % 182) * 0.0027777f + 0.5f);
                    }
                    else
                    {
                        GI.ZD_SetObjectScale(SpriteData[K].ID, 1.0f);
                    }

                    // Switch the Alpha channel accordingly
                    if (SWITCH_Opacity.SwitchStatus)
                    {
                        if (fl > 0.999) GI.ZD_SetObjectAlpha(SpriteData[K].ID, (byte)Math.Min(100 * scale, 255), GI.ZD_DRAW_DEFERRED);
                    }
                    else
                    {
                        GI.ZD_SetObjectAlpha(SpriteData[K].ID, 255, GI.ZD_DRAW_DEFERRED);
                    }

                    GI.ZD_SetObjectXY(SpriteData[K].ID, SpriteData[K].xPos, SpriteData[K].yPos, GI.ZD_DRAW_DEFERRED);

                    if (SWITCH_SelfRotation.SwitchStatus)
                    {
                        GI.ZD_SetObjectAngle(SpriteData[K].ID, (int)SpriteAngle[K], GI.ZD_DRAW_DEFERRED);
                    }
                    else
                    {
                        GI.ZD_SetObjectAngle(SpriteData[K].ID, 0, GI.ZD_DRAW_DEFERRED);
                    }
                }

                // Compute current Z-order
                Array.Sort(Zorder, FaceOrder);

                // Set the Sprite's Z-order
                K = FaceOrder[0];
                if (gbMoveToLeft)
                {
                    K += 1; if (K > UboundSpriteData - 1) K = 0;
                }
                GI.ZD_SetObjectZorder(SpriteData[K].ID, GI.ZD_ORDER_BOTTOM);
                GI.ZD_SetObjectZorder(SpriteData[FaceOrder[UboundSpriteData - 1]].ID, GI.ZD_ORDER_TOP);

                if (SWITCH_Marquee.SwitchStatus) DrawMarquee();

                // Update control display
                GI.UpdateWindow(hGDImageCtrl, false);
            }    
        }

        private void DrawMarquee()
        {
            int X, Y;
            GI.ZD_GetObjectXY(ID_OBJECT_TEXT, out X, out Y);
            X -= 2; if (X < -gnBoundWidth) X = GDImage_Template.Size.Width;
            GI.ZD_SetObjectXY(ID_OBJECT_TEXT, X, Y, GI.ZD_DRAW_DEFERRED);
        }

        private void Wallpaper_Click()
        {
            if (SWITCH_Wallpaper.SwitchStatus)
            {
                GI.ZI_SetFromFile(hGDImageCtrl, Api.ExePath() + "Back.jpg");
            }
            else
            {
                GI.ZI_SetFromFile(hGDImageCtrl, Api.ExePath() + "NoBackground.gif");
            }
        }

        private void PlayCarousel()
        {
            if (SpriteData[0].nWidth == 0)
            {
                LoadSprites();
                gbAnimate = true;
            }
            else
            {
                gbAnimate = true;
            }
            BTN_Play.Visible = false;
            BTN_Pause.Visible = true;
        }
        private void BTN_Play_Click(object sender, EventArgs e)
        {
            PlayCarousel();
        }

        private void BTN_Pause_Click(object sender, EventArgs e)
        {
            gbAnimate = false;
            BTN_Pause.Visible = false;
            BTN_Play.Visible = true;
        }

        private void BTN_Left_Click(object sender, EventArgs e)
        {
            gbMoveToLeft = true;
            PlayCarousel();
        }

        private void BTN_Right_Click(object sender, EventArgs e)
        {
            gbMoveToLeft = false;
            PlayCarousel();
        }

        private void SaveInitialValue()
        {
            Focal_InitialValue = POT_Focal.Value;
            Speed_InitialValue = POT_Speed.Value;
            Y_InitialValue = POT_Y.Value;
        }

        private void BTN_Reset_Click(object sender, EventArgs e)
        {
            POT_Focal.Value = Focal_InitialValue;
            POT_Speed.Value = Speed_InitialValue;
            POT_Y.Value = Y_InitialValue;
        }


    } // END class MAIN_Form

} // END namespace

